@@ -19,15 +19,24 @@
 
 #include <libgnomevfs/gnome-vfs.h>
 #include <vorbis/vorbisfile.h>
-#include <id3tag.h>
 #include <FLAC/metadata.h>
 #include <FLAC/stream_decoder.h>
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <config.h>
 
+#if HAVE_FAAD
+#include <faad.h>
+#include <mp4ff.h>
+#endif /* HAVE_FAAD */
+
+#if HAVE_ID3TAG
+#include <id3tag.h>
 #include "id3-vfs/id3-vfs.h"
+#endif /* HAVE_ID3TAG */
+
 #include "ogg-helper.h"
 #include "metadata.h"
 
@@ -43,6 +52,8 @@ struct _Metadata {
 	char *album;
 
 	int track_number;
+	int total_tracks;
+	int disc_number;
 
 	char *year;
 
@@ -58,6 +69,49 @@ struct _Metadata {
 	GdkPixbuf *album_art;
 };
 
+static void
+parse_raw_track_number (Metadata *metadata,
+			const char *raw)
+{
+	char *part2;
+
+	if (raw == NULL) {
+		metadata->track_number = -1;
+		metadata->total_tracks = -1;
+
+		return;
+	}
+
+	part2 = strstr (raw, "/");
+
+	if (part2 != NULL)
+		metadata->total_tracks = atoi (part2 + 1);
+	else
+		metadata->total_tracks = -1;
+
+	metadata->track_number = atoi (raw);
+}
+
+static void
+ensure_track_number (Metadata *metadata,
+		     const char *filename)
+{
+	char *basename;
+	int num;
+
+	if (metadata->track_number > 0)
+		return;
+
+	basename = g_path_get_basename (filename);
+	num = atoi (basename);
+	g_free (basename);
+
+	if (num < 100) /* protection against strange filenames */
+		metadata->track_number = num;
+}
+
+#if HAVE_ID3TAG
+
 static int
 get_mp3_duration (struct id3_tag *tag)
 {
@@ -87,7 +141,7 @@ get_mp3_duration (struct id3_tag *tag)
 	 * of the audio file in milliseconds,
 	 * represented as a numeric string."
 	 */
-	time = atol (latin1);
+	time = atol ((char *) latin1);
 	g_free (latin1);
 
 	if (time > 0)
@@ -112,6 +166,7 @@ get_mp3_picture_data (struct id3_tag *tag, const char *field_name)
 	id3_byte_t const *picture_data;
 	id3_length_t length;
 	GdkPixbuf *ret;
+	gboolean bail = FALSE;
 
 	/* Note: An MP3 can actualy have multiple embedded images. We should
 	 * really check them all for one of "type" [field 2] 03, however with
@@ -135,10 +190,16 @@ get_mp3_picture_data (struct id3_tag *tag, const char *field_name)
 
 	GdkPixbufLoader *pb_loader = gdk_pixbuf_loader_new ();
 	if (!gdk_pixbuf_loader_write (pb_loader, picture_data, length, NULL))
-		return NULL;
+		bail = TRUE;
 
 	if (!gdk_pixbuf_loader_close (pb_loader, NULL))
+		bail = TRUE;
+
+	if (bail) {
+		g_object_unref (pb_loader);
+
 		return NULL;
+	}
 
 	ret = gdk_pixbuf_loader_get_pixbuf (pb_loader);
 	if (ret != NULL &&
@@ -268,10 +329,13 @@ get_mp3_comment_value (struct id3_tag *tag,
 	if (utf8 == NULL)
 		return NULL;
 
-	if (!g_utf8_validate (utf8, -1, NULL))
-		return NULL;
+	if (!g_utf8_validate ((char *) utf8, -1, NULL)) {
+		g_free (utf8);
 
-	return utf8;
+		return NULL;
+	}
+
+	return (char *) utf8;
 }
 
 static Metadata *
@@ -285,6 +349,7 @@ assign_metadata_mp3 (const char *filename,
 	int bitrate, samplerate, channels, version, vbr, count, i;
 	int time, tag_time;
 	char *track_number_raw;
+	char *disc_number_raw;
 
 	file = id3_vfs_open (filename, ID3_FILE_MODE_READONLY);
 	if (file == NULL) {
@@ -345,12 +410,16 @@ assign_metadata_mp3 (const char *filename,
 	metadata->album = get_mp3_comment_value (tag, ID3_FRAME_ALBUM, 0);
 
 	track_number_raw = get_mp3_comment_value (tag, ID3_FRAME_TRACK, 0);
-	if (track_number_raw != NULL)
-		metadata->track_number = atoi (track_number_raw);
-	else
-		metadata->track_number = -1;
+	parse_raw_track_number (metadata, track_number_raw);
 	g_free (track_number_raw);
 
+        disc_number_raw = get_mp3_comment_value (tag, "TPOS", 0);
+        if (disc_number_raw != NULL)
+                metadata->disc_number = atoi (disc_number_raw);
+        else
+                metadata->disc_number = -1;
+        g_free (disc_number_raw);
+
 	metadata->year = get_mp3_comment_value (tag, ID3_FRAME_YEAR, 0);
 
 	metadata->gain = get_mp3_gain (tag);
@@ -364,6 +433,128 @@ assign_metadata_mp3 (const char *filename,
 	return metadata;
 }
 
+#endif /* HAVE_ID3TAG */
+
+#if HAVE_FAAD
+
+static uint32_t mp4_read_callback (void *user_data, void *buffer, uint32_t length);
+static uint32_t mp4_seek_callback (void *user_data, uint64_t position);
+
+static uint32_t
+mp4_read_callback (void *user_data, void *buffer, uint32_t length)
+{
+	GnomeVFSFileSize read;
+	gnome_vfs_read ((GnomeVFSHandle*)user_data, (gpointer *) buffer,  length, &read);
+	return (uint32_t)read;
+}
+
+static uint32_t
+mp4_seek_callback (void *user_data, uint64_t position)
+{
+	return (uint32_t) gnome_vfs_seek ((GnomeVFSHandle*) user_data, GNOME_VFS_SEEK_START, position);
+}
+
+static Metadata *
+assign_metadata_mp4 (const char *filename,
+		     char **error_message_return)
+{
+	Metadata *m = NULL;
+	GnomeVFSHandle *fh;
+	mp4ff_t *mp4f;
+	mp4ff_callback_t *mp4cb = (mp4ff_callback_t*) malloc (sizeof (mp4ff_callback_t));
+	gchar *value;
+	gchar *item;
+	unsigned char *buff = NULL;
+        guint buff_size = 0;
+	int j, k;
+	mp4AudioSpecificConfig mp4ASC;
+	long samples;
+	float f = 1024.0;
+
+	if (gnome_vfs_open (&fh, filename, GNOME_VFS_OPEN_READ) != GNOME_VFS_OK) {
+                *error_message_return = g_strdup ("Failed to open file for reading");
+                return NULL;
+        }
+
+	mp4cb->read = mp4_read_callback;
+	mp4cb->seek = mp4_seek_callback;
+	mp4cb->user_data = fh;
+
+	mp4f = mp4ff_open_read (mp4cb);
+
+	if (!mp4f) {
+		*error_message_return = g_strdup ("Unable to open the AAC file");
+		return NULL;
+	}
+
+	if (mp4ff_total_tracks (mp4f) > 1) {
+		 *error_message_return = g_strdup ("Multi-track AAC files not supported");
+		 return NULL;
+	}
+
+	m = g_new0 (Metadata, 1);
+
+	j = mp4ff_meta_get_num_items (mp4f);
+
+	for (k = 0; k < j; k++) {
+		if (mp4ff_meta_get_by_index (mp4f, k, &item, &value)) {
+			if (!strcmp (item, "title")) {
+				if (g_utf8_validate (value, -1, NULL))
+					m->title = g_strdup (value);
+			} else if (!strcmp (item, "artist")) {
+				if (g_utf8_validate (value, -1, NULL)) {
+					m->artists = g_new (char *, 2);
+					m->artists[0] = g_strdup (value);
+					m->artists[1] = NULL;
+					m->artists_count = 1;
+				}
+			} else if (!strcmp (item, "album")) {
+				if (g_utf8_validate (value, -1, NULL))
+					m->album = g_strdup (value);
+			} else if (!strcmp (item, "track"))
+				m->track_number = atoi (value);
+			else if (!strcmp (item, "totaltracks"))
+				m->total_tracks = atoi (value);
+			else if (!strcmp (item, "disc"))
+				m->disc_number = atoi (value);
+			else if (!strcmp (item, "date")) {
+				if (g_utf8_validate (value, -1, NULL))
+					m->year = g_strdup (value);
+			}
+			free (item);
+			free (value);
+		}
+	}
+
+	/*
+	 * duration code shameless based on code from frontend/main.c in faad2
+	 */
+
+	samples = mp4ff_num_samples (mp4f, 0);
+	mp4ff_get_decoder_config (mp4f, 0, &buff, &buff_size);
+
+	if (buff)
+	{
+		if (AudioSpecificConfig (buff, buff_size, &mp4ASC) >= 0)
+		{
+			free (buff);
+		        if (mp4ASC.sbr_present_flag == 1) {
+		            f = f * 2.0;
+		        }
+
+			m->duration = (float) samples * (float) (f - 1.0) / (float) mp4ASC.samplingFrequency;
+		}
+	}
+
+	mp4ff_close (mp4f);
+	free (mp4cb);
+	gnome_vfs_close (fh);
+
+	return m;
+}
+
+#endif /* HAVE_FAAD */
+
 static ov_callbacks file_info_callbacks =
 {
         ogg_helper_read,
@@ -431,10 +622,19 @@ assign_metadata_vorbiscomment (Metadata *metadata,
 	metadata->album = get_vorbis_comment_value (comment, "album", 0);
 
 	raw = vorbis_comment_query (comment, "tracknumber", 0);
+	parse_raw_track_number (metadata, raw);
+
+	if (metadata->total_tracks < 0) {
+		raw = vorbis_comment_query (comment, "tracktotal", 0);
+		if (raw != NULL)
+			metadata->total_tracks = atoi (raw);
+	}
+
+	raw = vorbis_comment_query (comment, "discnumber", 0);
 	if (raw != NULL)
-		metadata->track_number = atoi (raw);
+		metadata->disc_number = atoi (raw);
 	else
-		metadata->track_number = -1;
+		metadata->disc_number = -1;
 
 	metadata->year = get_vorbis_comment_value (comment, "date", 0);
 
@@ -669,19 +869,28 @@ metadata_load (const char *filename,
 	gnome_vfs_get_file_info (escaped, info,
 				 GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 
-	if (!strcmp (info->mime_type, "audio/x-mp3") ||
-	    !strcmp (info->mime_type, "audio/mpeg"))
-		m = assign_metadata_mp3 (escaped, info, error_message_return);
-	else if (!strcmp (info->mime_type, "application/x-ogg") ||
-		 !strcmp (info->mime_type, "application/ogg"))
+	if (!strcmp (info->mime_type, "application/x-ogg") ||
+	    !strcmp (info->mime_type, "application/ogg"))
 		m = assign_metadata_ogg (escaped, error_message_return);
+#if HAVE_ID3TAG
+	else if (!strcmp (info->mime_type, "audio/x-mp3") ||
+	         !strcmp (info->mime_type, "audio/mpeg"))
+		m = assign_metadata_mp3 (escaped, info, error_message_return);
+#endif /* HAVE_ID3TAG */
 	else if (!strcmp (info->mime_type, "application/x-flac") ||
 		 !strcmp (info->mime_type, "audio/x-flac"))
 		m = assign_metadata_flac (escaped, error_message_return);
+#if HAVE_FAAD
+	else if (!strcmp (info->mime_type, "application/x-m4a") ||
+		 !strcmp (info->mime_type, "audio/x-m4a"))
+		m = assign_metadata_mp4 (filename, error_message_return);
+#endif /* HAVE_FAAD */
 	else
 		*error_message_return = g_strdup ("Unknown format");
 
 	if (m != NULL) {
+		ensure_track_number (m, filename);
+
 		m->mime_type = g_strdup (info->mime_type);
 		m->mtime = info->mtime;
 	}
@@ -708,8 +917,7 @@ metadata_free (Metadata *metadata)
 	g_free (metadata->year);
 	g_free (metadata->mime_type);
 
-	/* do not unref the album_art pixbuf, as it becomes managed
-	 * in Metadata.cs */
+	g_object_unref (metadata->album_art);
 
 	g_free (metadata);
 }
@@ -767,7 +975,10 @@ metadata_get_album_art (Metadata *metadata)
 {
 	g_return_val_if_fail (metadata != NULL, NULL);
 
-	return metadata->album_art;
+	if (metadata->album_art != NULL)
+		return g_object_ref (metadata->album_art);
+	else
+		return NULL;
 }
 
 int
@@ -778,6 +989,22 @@ metadata_get_track_number (Metadata *metadata)
 	return metadata->track_number;
 }
 
+int
+metadata_get_total_tracks (Metadata *metadata)
+{
+	g_return_val_if_fail (metadata != NULL, -1);
+
+	return metadata->total_tracks;
+}
+
+int
+metadata_get_disc_number (Metadata *metadata)
+{
+	g_return_val_if_fail (metadata != NULL, -1);
+
+	return metadata->disc_number;
+}
+
 const char *
 metadata_get_year (Metadata *metadata)
 {
