@@ -218,7 +218,7 @@ pointer_list_model_row_draggable (GtkTreeDragSource *drag_source,
 {
   g_return_val_if_fail (IS_POINTER_LIST_MODEL (drag_source), FALSE);
 
-  return (POINTER_LIST_MODEL (drag_source)->sort_func == NULL);
+  return TRUE;
 }
   
 static gboolean
@@ -251,118 +251,12 @@ pointer_list_model_drag_data_get (GtkTreeDragSource *drag_source,
   return FALSE;
 }
 
-static void
-put_before (PointerListModel *model,
-            GSequencePtr      ptr,
-            GSequencePtr      before)
-{
-  GSequence *pointers;
-  GSequencePtr *old_order;
-  GtkTreePath *path;
-  int *new_order;
-  int length;
-  int i;
-
-  if (ptr == before)
-    return;
-
-  pointers = model->pointers;
-  length = g_sequence_get_length (pointers);
-
-  if (length <= 1)
-    return;
- 
-  /* Generate old order of GSequencePtrs. */
-  old_order = g_new (GSequencePtr, length);
-  for (i = 0; i < length; ++i)
-    old_order[i] = g_sequence_get_ptr_at_pos (pointers, i);
-
-  g_sequence_ptr_move_before (ptr, before);
-
-  model->moved_pointer = ptr;
-
-  /* Generate new order. */
-  new_order = g_new (int, length);
-  for (i = 0; i < length; ++i)
-    new_order[i] = g_sequence_ptr_get_position (old_order[i]);
-
-  path = gtk_tree_path_new ();
-  
-  gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, NULL, new_order);
-  
-  gtk_tree_path_free (path);
-  g_free (old_order);
-  g_free (new_order);
-}
-
 static gboolean
 pointer_list_model_drag_data_received (GtkTreeDragDest   *drag_dest,
-                                     GtkTreePath       *dest,
-                                     GtkSelectionData  *selection_data)
+                                       GtkTreePath       *dest,
+                                       GtkSelectionData  *selection_data)
 {
-  GtkTreeModel *tree_model;
-  PointerListModel *model;
-  GtkTreeModel *src_model = NULL;
-  GtkTreePath *src_path = NULL;
-  gboolean retval = FALSE;
-
-  g_return_val_if_fail (IS_POINTER_LIST_MODEL (drag_dest), FALSE);
-
-  tree_model = GTK_TREE_MODEL (drag_dest);
-  model = POINTER_LIST_MODEL (drag_dest);
-
-  if (gtk_tree_get_row_drag_data (selection_data,
-				  &src_model,
-				  &src_path) &&
-      src_model == tree_model)
-    {
-      /* Copy the given row to a new position */
-      GtkTreeIter src_iter;
-      GtkTreeIter dest_iter;
-      GtkTreePath *prev;
-
-      if (!gtk_tree_model_get_iter (src_model,
-                                    &src_iter,
-                                    src_path))
-        {
-          goto out;
-        }
-
-      /* Get the path to insert _after_ (dest is the path to insert _before_) */
-      prev = gtk_tree_path_copy (dest);
-
-      if (!gtk_tree_path_prev (prev))
-        {
-          put_before (model, src_iter.user_data,
-		      g_sequence_get_begin_ptr (model->pointers));
-
-          retval = TRUE;
-        }
-      else
-        {
-          if (gtk_tree_model_get_iter (tree_model, &dest_iter, dest))
-            {
-              put_before (model, src_iter.user_data,
-			  dest_iter.user_data);
-            }
-	  else
-            {
-              put_before (model, src_iter.user_data,
-		          g_sequence_get_end_ptr (model->pointers));
-            }
-
-	  retval = TRUE;
-        }
-
-      gtk_tree_path_free (prev);
-    }
-
- out:
-
-  if (src_path)
-    gtk_tree_path_free (src_path);
-
-  return retval;
+  return FALSE;
 }
 
 static gboolean
@@ -453,7 +347,6 @@ pointer_list_model_init (PointerListModel *model)
   model->reverse_map = g_hash_table_new (NULL, NULL);
   model->stamp = g_random_int ();
   model->sort_func = NULL;
-  model->sort_type = GTK_SORT_ASCENDING;
 
   model->current_pointer = NULL;
 }
@@ -550,6 +443,54 @@ pointer_list_model_add (PointerListModel *model, gpointer pointer)
   return TRUE;
 }
 
+gboolean
+pointer_list_model_insert (PointerListModel *model, gpointer pointer,
+			   gpointer ins, GtkTreeViewDropPosition pos)
+{
+  GtkTreeIter iter;
+  GtkTreePath *path;
+  GSequencePtr new_ptr, before_ptr;
+  gboolean is_end;
+
+  if (g_hash_table_lookup (model->reverse_map, pointer))
+    return FALSE;
+
+  before_ptr = g_hash_table_lookup (model->reverse_map, ins);
+  g_assert (before_ptr != NULL);
+
+  is_end = g_sequence_ptr_is_end (g_sequence_ptr_next (before_ptr));
+
+  new_ptr = g_sequence_append (model->pointers, pointer);
+
+  switch (pos) {
+    case GTK_TREE_VIEW_DROP_BEFORE:
+    case GTK_TREE_VIEW_DROP_INTO_OR_BEFORE:
+      break;
+    case GTK_TREE_VIEW_DROP_AFTER:
+    case GTK_TREE_VIEW_DROP_INTO_OR_AFTER:
+      if (!is_end)
+        before_ptr = g_sequence_ptr_next (before_ptr);
+      else
+        before_ptr = NULL;
+
+      break;
+  }
+
+  if (before_ptr != NULL)
+    g_sequence_ptr_move_before (new_ptr, before_ptr);
+
+  g_hash_table_insert (model->reverse_map, pointer, new_ptr);
+
+  iter.stamp = model->stamp;
+  iter.user_data = new_ptr;
+
+  path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+  gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
+  gtk_tree_path_free (path);
+
+  return TRUE;
+}
+
 void
 pointer_list_model_remove_iter (PointerListModel *model, GtkTreeIter *iter)
 {
@@ -597,8 +538,8 @@ pointer_list_model_clear (PointerListModel *model)
     }
 }
 
-static void
-pointer_list_model_sort (PointerListModel *model)
+void
+pointer_list_model_sort (PointerListModel *model, GCompareDataFunc sort_func)
 {
   GSequence *pointers;
   GSequencePtr *old_order;
@@ -618,8 +559,8 @@ pointer_list_model_sort (PointerListModel *model)
   for (i = 0; i < length; ++i)
     old_order[i] = g_sequence_get_ptr_at_pos (pointers, i);
 
-  g_sequence_sort (pointers, (GCompareDataFunc) model->sort_func, NULL);
-  
+  g_sequence_sort (pointers, sort_func, NULL);
+
   /* Generate new order. */
   new_order = g_new (int, length);
   for (i = 0; i < length; ++i)
@@ -636,16 +577,14 @@ pointer_list_model_sort (PointerListModel *model)
 
 void
 pointer_list_model_set_sorting (PointerListModel  *model,
-			      GCompareFunc sort_func,
-			      GtkSortType      type)
+			        GCompareFunc sort_func)
 {
-  if (sort_func == model->sort_func && type == model->sort_type)
+  if (sort_func == model->sort_func)
     return;
 
   model->sort_func = sort_func;
-  model->sort_type = type;
 
-  pointer_list_model_sort (model);
+  pointer_list_model_sort (model, (GCompareDataFunc) sort_func);
 }
 
 gboolean
@@ -669,7 +608,7 @@ pointer_list_model_pointer_get_iter (PointerListModel *model,
 }
 
 gpointer
-pointer_list_model_get_pointer (PointerListModel *model, GtkTreeIter *iter)
+pointer_list_model_iter_get_pointer (PointerListModel *model, GtkTreeIter *iter)
 {
   g_return_val_if_fail (model->stamp == iter->stamp, NULL);
   
@@ -960,14 +899,3 @@ pointer_list_model_has_first (PointerListModel *model)
 
   return (len > 0);
 }
-
-void
-pointer_list_model_get_moved_iter (PointerListModel *model,
-				   GtkTreeIter      *iter)
-{
-  g_return_if_fail (IS_POINTER_LIST_MODEL (model));
-  g_return_if_fail (iter != NULL);
-
-  iter->stamp = model->stamp;
-  iter->user_data = model->moved_pointer;
-}
