@@ -39,11 +39,14 @@ namespace Muine
 		// Strings
 		private static readonly string string_title = 
 			Catalog.GetString ("Play Album");
+
 		private static readonly string string_artists = 
 			Catalog.GetString ("Performed by {0}");
 
-		// DnD Targets	
-		private static TargetEntry [] source_entries = new TargetEntry [] {
+		// Static
+		// Static :: Objects
+		// Static :: Objects :: DnD Targets
+		private static TargetEntry [] source_entries = {
 			DndUtils.TargetMuineAlbumList,
 			DndUtils.TargetUriList
 		};
@@ -57,6 +60,12 @@ namespace Muine
 		private int pixbuf_column_width = CoverDatabase.CoverSize + (5 * 2);
 
 		// Constructor
+		/// <summary>
+		///	Creates a new Add Album window.
+		/// </summary>
+		/// <remarks>
+		///	This is created when "Play Album" is clicked.
+		/// </remarks>
 		public AddAlbumWindow ()
 		{
 			base.Title = string_title;
@@ -71,10 +80,10 @@ namespace Muine
 			TreeViewColumn col = new TreeViewColumn ();
 			col.Sizing = TreeViewColumnSizing.Fixed;
 			col.Spacing = 4;
-			col.PackStart (pixbuf_renderer, false);
-			col.PackStart (base.TextRenderer, true);
-			col.SetCellDataFunc (pixbuf_renderer, new TreeCellDataFunc (PixbufCellDataFunc));
-			col.SetCellDataFunc (base.TextRenderer, new TreeCellDataFunc (TextCellDataFunc));
+			col.PackStart (pixbuf_renderer  , false);
+			col.PackStart (base.TextRenderer, true );
+			col.SetCellDataFunc (pixbuf_renderer  , new TreeCellDataFunc (PixbufCellDataFunc));
+			col.SetCellDataFunc (base.TextRenderer, new TreeCellDataFunc (TextCellDataFunc  ));
 			base.List.AppendColumn (col);
 
 			base.List.DragSource = source_entries;
@@ -98,6 +107,9 @@ namespace Muine
 		// Methods
 		// Methods :: Private
 		// Methods :: Private :: EnableDragDest
+		/// <summary>
+		/// 	Turns on Drag-and-Drop.
+		/// </summary>
 		private void EnableDragDest ()
 		{
 			if (drag_dest_enabled)
@@ -105,13 +117,19 @@ namespace Muine
 
 			base.List.DragDataReceived += new DragDataReceivedHandler (OnDragDataReceived);
 			Gtk.Drag.DestSet (base.List, DestDefaults.All,
-					  CoverImage.DragEntries, Gdk.DragAction.Copy);
+				CoverImage.DragEntries, Gdk.DragAction.Copy);
 
 			drag_dest_enabled = true;
 		}
 
 		// Handlers
 		// Handlers :: OnCoversDoneLoading
+		/// <csummary>
+		/// 	Handler called when the album covers are done loading.
+		/// </summary>
+		/// <remarks>
+		///	Enables Drag-and-Drop and redraws the list.
+		/// </remarks>
 		private void OnCoversDoneLoading ()
 		{
 			EnableDragDest ();
@@ -120,10 +138,15 @@ namespace Muine
 		}
 
 		// Handlers :: OnDragDataReceived
+		/// <summary>
+		/// 	Handler called when Drag-and-Drop data is received.
+		/// </summary>
+		/// <remarks>
+		/// 	External covers may be Drag-and-Dropped onto an album.
+		/// </remarks>
 		private void OnDragDataReceived (object o, DragDataReceivedArgs args)
 		{
 			TreePath path;
-
 			if (!base.List.GetPathAtPos (args.X, args.Y, out path))
 				return;
 
@@ -134,6 +157,12 @@ namespace Muine
 		}
 
 		// Handlers :: OnDragDataGet
+		/// <summary>
+		/// 	Handler to be activated when Drag-and-Drop data is requested.
+		/// </summary>
+		/// <remarks>
+		/// 	Albums may be copied by dragging them to Nautilus.
+		/// </remarks>
 		private void OnDragDataGet (object o, DragDataGetArgs args)
 		{
 			List albums = base.List.SelectedHandles;
@@ -151,7 +180,7 @@ namespace Muine
 				}
 		
 				args.SelectionData.Set (Gdk.Atom.Intern (DndUtils.TargetUriList.Target, false),
-							8, System.Text.Encoding.UTF8.GetBytes (files));
+					8, System.Text.Encoding.UTF8.GetBytes (files));
 							
 				break;
 
@@ -164,7 +193,7 @@ namespace Muine
 				}
 				
 				args.SelectionData.Set (Gdk.Atom.Intern (DndUtils.TargetMuineAlbumList.Target, false),
-							8, System.Text.Encoding.ASCII.GetBytes (ptrs));
+					8, System.Text.Encoding.ASCII.GetBytes (ptrs));
 							
 				break;
 
@@ -174,23 +203,41 @@ namespace Muine
 		}
 
 		// Handlers :: OnAdded
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		/// <summary>
+		/// 	Handler called when an album is added.
+		/// </summary>		
+		/// <param name="album">
+		///	The <see cref="Album" /> that was added.
+		/// </param>
 		protected void OnAdded (Album album)
 		{
 			base.List.HandleAdded (album.Handle, 
-					       album.FitsCriteria (base.Entry.SearchBits));
+				album.FitsCriteria (base.Entry.SearchBits));
 		}
 
 		// Handlers :: OnChanged
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		/// <summary>
+		/// 	Handler called when an album is changed.
+		/// </summary>
+		/// <param name="album">
+		///	The <see cref="Album" /> that was changed.
+		/// </param>
 		protected void OnChanged (Album album)
 		{
 			base.List.HandleChanged (album.Handle, 
-						 album.FitsCriteria (base.Entry.SearchBits));
+				album.FitsCriteria (base.Entry.SearchBits));
 		}
 
 		// Handlers :: OnRemoved
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		/// <summary>
+		/// 	Handler called when an album is removed.
+		/// </summary>
+		/// <param name="album">
+		///	The <see cref="Album" /> that was removed.
+		/// </param>
 		protected void OnRemoved (Album album)
 		{
 			base.List.HandleRemoved (album.Handle);
@@ -198,6 +245,20 @@ namespace Muine
 
 		// Delegate Functions		
 		// Delegate Functions :: SortFunc
+		/// <summary>
+		/// 	Delegate used in sorting the album list.
+		/// </summary>
+		/// <param name="a_ptr">
+		///	Handler for first <see cref="Album" />.
+		/// </param>
+		/// <param name="b_ptr">
+		///	Handler for second <see cref="Album" />.
+		/// </param>
+		/// <returns>
+		///	The result of comparing the albums with
+		///	<see cref="Item.CompareTo" />.
+		/// </returns>
+		/// <seealso cref="Item.CompareTo" />
 		private int SortFunc (IntPtr a_ptr, IntPtr b_ptr)
 		{
 			Album a = Album.FromHandle (a_ptr);
@@ -207,22 +268,55 @@ namespace Muine
 		}
 
 		// Delegate Functions :: PixbufCellDataFunc
+		/// <summary>
+		/// 	Delegate used to render the covers.
+		/// </summary>
+		/// <param name="col">
+		///	A <see cref="Gtk.TreeViewColumn" />.
+		/// </param>
+		/// <param name="cell">
+		///	A <see cref="Gtk.CellRenderer" />.
+		/// </param>
+		/// <param name="model">
+		///	A <see cref="Gtk.TreeModel" />.
+		/// </param>
+		/// <param name="iter">
+		///	A <see cref="Gtk.TreeIter" />.
+		/// </param>
 		private void PixbufCellDataFunc (TreeViewColumn col, CellRenderer cell,
-					         TreeModel model, TreeIter iter)
+						 TreeModel model, TreeIter iter)
 		{
 			CellRendererPixbuf r = (CellRendererPixbuf) cell;
 			Album album = Album.FromHandle (base.List.Model.HandleFromIter (iter));
 
-			r.Pixbuf = (album.CoverImage != null)
-				? album.CoverImage
-				: (Global.CoverDB.Loading)
-					? Global.CoverDB.DownloadingPixbuf
-					: nothing_pixbuf;
+			if (album.CoverImage != null)
+				r.Pixbuf = album.CoverImage;
+
+			else if (Global.CoverDB.Loading)
+				r.Pixbuf = Global.CoverDB.DownloadingPixbuf;
+
+			else
+				r.Pixbuf = nothing_pixbuf;
 
 			r.Width = r.Height = pixbuf_column_width;
 		}
 
 		// Delegate Functions :: TextCellDataFunc
+		/// <summary>
+		/// 	Delegate used to render the album text.
+		/// </summary>		
+		/// <param name="col">
+		///	A <see cref="Gtk.TreeViewColumn" />.
+		/// </param>
+		/// <param name="cell">
+		///	A <see cref="Gtk.CellRenderer" />.
+		/// </param>
+		/// <param name="model">
+		///	A <see cref="Gtk.TreeModel" />.
+		/// </param>
+		/// <param name="iter">
+		///	A <see cref="Gtk.TreeIter" />.
+		/// </param>
 		private void TextCellDataFunc (TreeViewColumn col, CellRenderer cell,
 					       TreeModel model, TreeIter iter)
 		{
@@ -231,12 +325,13 @@ namespace Muine
 
 			string performers = "";
 			if (album.Performers.Length > 0)
-				performers = StringUtils.EscapeForPango (String.Format (string_artists, StringUtils.JoinHumanReadable (album.Performers, 2)));
+				performers = StringUtils.EscapeForPango (String.Format (string_artists, 
+					StringUtils.JoinHumanReadable (album.Performers, 2)));
 
 			r.Markup = String.Format ("<b>{0}</b>\n{1}\n\n{2}",
-						  StringUtils.EscapeForPango (album.Name),
-						  StringUtils.EscapeForPango (StringUtils.JoinHumanReadable (album.Artists, 3)),
-						  performers);
+				StringUtils.EscapeForPango (album.Name),
+				StringUtils.EscapeForPango (StringUtils.JoinHumanReadable (album.Artists, 3)),
+				performers);
 		}
 	}
 }
