@@ -40,19 +40,27 @@ namespace Muine
 		private static readonly string string_title = 
 			Catalog.GetString ("Play Song");
 
-		// DnD targets	
-		private static TargetEntry [] source_entries = new TargetEntry [] {
+		// Static
+		// Static :: Objects
+		// Static :: Objects :: DnD targets
+		private static TargetEntry [] source_entries = {
 			DndUtils.TargetMuineSongList,
 			DndUtils.TargetUriList
 		};
 
-		// Constructor	
+		// Constructor
+		/// <summary>
+		///	Creates a new Add Song window.
+		/// </summary>
+		/// <remarks>
+		///	This is created when "Play Song" is clicked.
+		/// </remarks>
 		public AddSongWindow ()
 		{
 			base.Title = string_title;
 
 			base.SetGConfSize (GConfKeyWidth , GConfDefaultWidth, 
-					   GConfKeyHeight, GConfDefaultHeight);
+				GConfKeyHeight, GConfDefaultHeight);
 
 			base.Items = Global.DB.Songs.Values;
 						
@@ -67,7 +75,7 @@ namespace Muine
 			base.List.DragSource = source_entries;
 			base.List.DragDataGet += new DragDataGetHandler (OnDragDataGet);
 
-			// Requires Mono 1.1+:
+			// FIXME: Requires Mono 1.1+:
 			// Global.DB.SongAdded   += new SongDatabase.SongAddedHandler   (base.OnAdded  );
 			// Global.DB.SongChanged += new SongDatabase.SongChangedHandler (base.OnChanged);
 			// Global.DB.SongRemoved += new SongDatabase.SongRemovedHandler (base.OnRemoved);
@@ -78,7 +86,13 @@ namespace Muine
 		}
 
 		// Handlers
-		// Handlers :: OnDragDataGet
+		// Handlers :: OnDragDataGet (Gtk.DragDataGetHandler)
+		/// <summary>
+		/// 	Handler to be activated when Drag-and-Drop data is requested.
+		/// </summary>
+		/// <remarks>
+		/// 	Songs may be copied by dragging them to Nautilus.
+		/// </remarks>
 		private void OnDragDataGet (object o, DragDataGetArgs args)
 		{
 			List songs = base.List.SelectedHandles;
@@ -93,7 +107,7 @@ namespace Muine
 				}
 		
 				args.SelectionData.Set (Gdk.Atom.Intern (DndUtils.TargetUriList.Target, false),
-							8, System.Text.Encoding.UTF8.GetBytes (files));
+					8, System.Text.Encoding.UTF8.GetBytes (files));
 							
 				break;	
 				
@@ -106,7 +120,7 @@ namespace Muine
 				}
 				
 				args.SelectionData.Set (Gdk.Atom.Intern (DndUtils.TargetMuineSongList.Target, false),
-						        8, System.Text.Encoding.ASCII.GetBytes (ptrs));
+					8, System.Text.Encoding.ASCII.GetBytes (ptrs));
 
 				break;
 
@@ -116,23 +130,43 @@ namespace Muine
 		}
 
 		// Handlers :: OnAdded
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		//	TODO: Change 'Song' parameter to 'song'
+		/// <summary>
+		/// 	Handler called when an song is added.
+		/// </summary>
+		/// <param name="song">
+		///	The <see cref="Song" /> that was added.
+		/// </param>
 		protected void OnAdded (Song Song)
 		{
 			base.List.HandleAdded (Song.Handle, 
-					       Song.FitsCriteria (base.Entry.SearchBits));
+				Song.FitsCriteria (base.Entry.SearchBits));
 		}
 
 		// Handlers :: OnSongChanged
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		//	TODO: Change 'Song' parameter to 'song'
+		/// <summary>
+		/// 	Handler called when an song is changed.
+		/// </summary>
+		/// <param name="song">
+		///	The <see cref="Song" /> that was changed.
+		/// </param>
 		protected void OnChanged (Song Song)
 		{
 			base.List.HandleChanged (Song.Handle, 
-						 Song.FitsCriteria (base.Entry.SearchBits));
+				Song.FitsCriteria (base.Entry.SearchBits));
 		}
 
 		// Handlers :: OnSongRemoved
-		// 	Remove if we depend on Mono 1.1+
+		// 	FIXME: Remove if we depend on Mono 1.1+
+		/// <summary>
+		/// 	Handler called when an song is removed.
+		/// </summary>
+		/// <param name="song">
+		///	The <see cref="Song" /> that was removed.
+		/// </param>
 		protected void OnRemoved (Song Song)
 		{
 			base.List.HandleRemoved (Song.Handle);
@@ -140,6 +174,20 @@ namespace Muine
 
 		// Delegate Functions
 		// Delegate Functions :: SortFunc		
+		/// <summary>
+		/// 	Delegate used in sorting the song list.
+		/// </summary>		
+		/// <param name="a_ptr">
+		///	Handler for first <see cref="Song" />.
+		/// </param>
+		/// <param name="b_ptr">
+		///	Handler for second <see cref="Song" />.
+		/// </param>
+		/// <returns>
+		///	The result of comparing the songs with
+		///	<see cref="Item.CompareTo" />.
+		/// </returns>
+		/// <seealso cref="Item.CompareTo" />
 		private int SortFunc (IntPtr a_ptr, IntPtr b_ptr)
 		{
 			Song a = Song.FromHandle (a_ptr);
@@ -149,6 +197,21 @@ namespace Muine
 		}
 
 		// Delegate Functions :: CellDataFunc
+		/// <summary>
+		/// 	Delegate used to render the song text.
+		/// </summary>		
+		/// <param name="col">
+		///	A <see cref="Gtk.TreeViewColumn" />.
+		/// </param>
+		/// <param name="cell">
+		///	A <see cref="Gtk.CellRenderer" />.
+		/// </param>
+		/// <param name="model">
+		///	A <see cref="Gtk.TreeModel" />.
+		/// </param>
+		/// <param name="iter">
+		///	A <see cref="Gtk.TreeIter" />.
+		/// </param>
 		private void CellDataFunc (TreeViewColumn col, CellRenderer cell,
 					   TreeModel model, TreeIter iter)
 		{
@@ -156,8 +219,8 @@ namespace Muine
 			Song song = Song.FromHandle (base.List.Model.HandleFromIter (iter));
 
 			r.Markup = String.Format ("<b>{0}</b>\n{1}",
-			                          StringUtils.EscapeForPango (song.Title),
-						  StringUtils.EscapeForPango (StringUtils.JoinHumanReadable (song.Artists)));
+				StringUtils.EscapeForPango (song.Title),
+				StringUtils.EscapeForPango (StringUtils.JoinHumanReadable (song.Artists)));
 		}
 	}
 }
