@@ -32,179 +32,12 @@ namespace Muine
 	public class CoverGetter
 	{
 		// GConf
-		private const string GConfKeyAmazonLocale = "/apps/muine/amazon_locale";
+		private const string GConfKeyAmazonLocale     = "/apps/muine/amazon_locale";
 		private const string GConfDefaultAmazonLocale = "us";
 
 		// Delegates
 		public delegate void GotCoverDelegate (Pixbuf pixbuf);
-
-		// Internal Classes
-		// Internal Classes :: GetWebThread
-		// 	TODO: Split off? This is big.
-		private class GetWebThread
-		{
-			// Delegates
-			private GotCoverDelegate done_func;
-			
-			// Objects
-			private CoverGetter getter;
-			private Pixbuf pixbuf = null;
-
-			// Variables
-			private string key;
-			private string url;
-
-			// Constructor
-			public GetWebThread (CoverGetter getter, string key, string url,
-					     GotCoverDelegate done_func)
-			{
-				this.getter = getter;
-				this.key = key;
-				this.url = url;
-				this.done_func = done_func;
-
-				Thread thread = new Thread (new ThreadStart (ThreadFunc));
-				thread.IsBackground = true;
-				thread.Priority = ThreadPriority.BelowNormal;
-				thread.Start ();
-			}
-
-			// Methods
-			// Methods :: Private
-			// Methods :: Private :: SignalIdle
-			private bool SignalIdle ()
-			{
-				// This is not entirely safe, as the user can in theory
-				// have changed the cover image between the thread and
-				// this idle. However, chances are very, very slim and
-				// things won't explode if it happens.
-				done_func (pixbuf);
-
-				return false;
-			}			
-
-			// Delegate Functions
-			// Delegate Functions :: ThreadFunc
-			private void ThreadFunc ()
-			{
-				try {
-					pixbuf = getter.Download (url);
-					pixbuf = getter.AddBorder (pixbuf);
-				} catch {}
-
-				if (Global.CoverDB.Covers [key] != null) {
-					// has been modified while we were downloading ..
-					return;
-				}
-
-				if (pixbuf != null)
-					Global.CoverDB.SetCover (key, pixbuf);
-
-				// Also do this if it is null, as we need to remove the
-				// downloading image ..
-				GLib.IdleHandler idle = new GLib.IdleHandler (SignalIdle);
-				GLib.Idle.Add (idle);
-			}
-		}
-
-		// Internal Classes :: GetAmazonThread
-		// 	TODO: Split off? This is big.
-		private class GetAmazonThread
-		{
-			// Objects
-			private CoverGetter getter;
-			private Queue queue;
-
-			// Internal Classes
-			// Internal Classes :: IdleData
-			// 	XXX: Internal classes inside internal classes? ick!
-			private class IdleData
-			{
-				private Album album;
-				private Pixbuf pixbuf;
-
-				// Constructor
-				public IdleData (Album album, Pixbuf pixbuf)
-				{
-					this.album = album;
-					this.pixbuf = pixbuf;
-
-					GLib.IdleHandler idle = new GLib.IdleHandler (IdleFunc);
-					GLib.Idle.Add (idle);
-				}
-
-				// Delegate Functions
-				// Delegate Functions :: IdleFunc
-				private bool IdleFunc ()
-				{
-					// This is not entirely safe, as the user can in theory
-					// have changed the cover image between the thread and
-					// this idle. However, chances are very, very slim and
-					// things won't explode if it happens.
-					album.CoverImage = pixbuf;
-		
-					return false;
-				}
-			}
-
-			// Constructor
-			public GetAmazonThread (CoverGetter getter)
-			{
-				this.getter = getter;
-
-				queue = Queue.Synchronized (new Queue ());
-
-				Thread thread = new Thread (new ThreadStart (ThreadFunc));
-				thread.IsBackground = true;
-				thread.Priority = ThreadPriority.BelowNormal;
-				thread.Start ();
-			}
-
-			// Properties
-			// Properties :: Queue (get;)
-			public Queue Queue {
-				get { return queue; }
-			}
-
-			// Delegate Functions
-			// Delegate Functions :: ThreadFunc
-			private void ThreadFunc ()
-			{
-				while (true) {
-					while (queue.Count > 0) {
-						Album album = (Album) queue.Dequeue ();
-						Pixbuf pixbuf = null;
-
-						try {
-							pixbuf = getter.DownloadFromAmazon (album);
-						} catch (WebException e) {
-							// Temporary web problem (Timeout etc.) - re-queue
-							Thread.Sleep (60000); // wait for a minute first
-							queue.Enqueue (album);
-							continue;
-						} catch (Exception e) {}
-
-						string key = album.Key;
-						
-						if (Global.CoverDB.Covers [key] != null) {
-							// has been modified while we were downloading ..
-							continue;
-						}
-
-						if (pixbuf != null) {
-							pixbuf = getter.AddBorder (pixbuf);
-							Global.CoverDB.SetCover (key, pixbuf);
-						} else
-							Global.CoverDB.UnmarkAsBeingChecked (key);
-
-						new IdleData (album, pixbuf);
-					}
-
-					Thread.Sleep (1000);
-				}
-			}						
-		}
-																
+														
 		// Objects
 		private CoverDatabase db;
 		private GnomeProxy proxy;
@@ -213,27 +46,37 @@ namespace Muine
 		// Variables
 		private string amazon_locale;
 
+		// Variables :: Cover Filenames
+		//	TODO: Maybe make checking these case-insensitve instead
+		//	and split possible extensions from possible filenames.
 		private string [] cover_filenames = {
-			"cover.jpg",
-			"Cover.jpg",
-			"cover.jpeg",
-			"Cover.jpeg",
-			"cover.png",
-			"Cover.png",
-			"folder.jpg",
-			"Folder.jpg",
-			"cover.gif",
-			"Cover.gif"
+			"cover.jpg" , "Cover.jpg" ,
+			"cover.jpeg", "Cover.jpeg",
+			"cover.png" , "Cover.png" ,
+			"cover.gif" , "Cover.gif" ,
+			"folder.jpg", "Folder.jpg"
 		};
 
 		// Constructor
+		/// <summary>
+		///	Creates a new <see cref="CoverGetter" />.
+		/// </summary>
+		/// <remarks>
+		///	This object is used to retrieve album covers in a 
+		///	variety of ways.
+		/// </remarks>
+		/// <param name="db">
+		///	The <see cref="CoverDatabase" /> which stores the covers.
+		/// </param>
 		public CoverGetter (CoverDatabase db)
 		{
 			this.db = db;
 			
-			amazon_locale = (string) Config.Get (GConfKeyAmazonLocale, GConfDefaultAmazonLocale);
+			amazon_locale = (string) Config.Get (GConfKeyAmazonLocale, 
+				GConfDefaultAmazonLocale);
+
 			Config.AddNotify (GConfKeyAmazonLocale,
-					  new GConf.NotifyEventHandler (OnAmazonLocaleChanged));
+				new GConf.NotifyEventHandler (OnAmazonLocaleChanged));
 
 			proxy = new GnomeProxy ();
 
@@ -243,9 +86,27 @@ namespace Muine
 		// Methods
 		// Methods :: Public
 		// Methods :: Public :: GetLocal
+		/// <summary>
+		///	Set the cover to a local file.
+		/// </summary>
+		/// <remarks>
+		///	Image is scaled and placed on a background with
+		///	<see cref="AddBorder" />.
+		/// </remarks>
+		/// <param name="key">
+		///	Album key.
+		/// </param>
+		/// <param name="file">
+		///	Filename.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" />.
+		/// </returns>
+		/// <exception cref="GLib.GException">
+		///	Thrown if loading file fails.
+		/// </exception>
 		public Pixbuf GetLocal (string key, string file)
 		{
-			// May throw an exception
 			Pixbuf pix = new Pixbuf (file);
 			pix = AddBorder (pix);
 			db.SetCover (key, pix);
@@ -253,6 +114,23 @@ namespace Muine
 		}
 
 		// Methods :: Public :: GetEmbedded
+		/// <summary>
+		///	Set the cover.
+		/// </summary>
+		/// <remarks>
+		///	Image is scaled and placed on a background with
+		///	<see cref="AddBorder" />.
+		/// </remarks>
+		/// <param name="key">
+		///	Album key.
+		/// </param>
+		/// <param name="pixbuf">
+		///	The <see cref="Gdk.Pixbuf" /> to use as the
+		///	album cover.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" />.
+		/// </returns>
 		public Pixbuf GetEmbedded (string key, Pixbuf pixbuf)
 		{
 			Pixbuf pix = AddBorder (pixbuf);
@@ -261,30 +139,67 @@ namespace Muine
 		}
 
 		// Methods :: Public :: GetFolderImage
+		/// <summary>
+		///	Search for the album cover in a folder.
+		/// </summary>
+		/// <remarks>
+		///	Image is scaled and placed on a background with
+		///	<see cref="AddBorder" />.
+		/// </remarks>
+		/// <param name="key">
+		///	Album key.
+		/// </param>
+		/// <param name="folder">
+		///	The folder in which to search.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> if a cover is found, null otherwise.
+		/// </returns>
 		public Pixbuf GetFolderImage (string key, string folder)
 		{
 			foreach (string fn in cover_filenames) {
 				FileInfo cover = new FileInfo (Path.Combine (folder, fn));
 				
-				if (cover.Exists) {
-					Pixbuf pix;
+				if (!cover.Exists)
+					continue;
 
-					try {
-						pix = new Pixbuf (cover.FullName);
-					} catch {
-						continue;
-					}
+				Pixbuf pix;
 
-					pix = AddBorder (pix);
-					db.SetCover (key, pix);
-					return pix;
+				try {
+					pix = new Pixbuf (cover.FullName);
+				} catch {
+					continue;
 				}
+
+				pix = AddBorder (pix);
+				db.SetCover (key, pix);
+				return pix;
 			}
 
 			return null;
 		}
 
 		// Methods :: Public :: GetWeb
+		/// <summary>
+		///	Get the cover from a URL.
+		/// </summary>
+		/// <remarks>
+		///	Immediately returns a temporary cover.
+		///	The actual downloading occurs in
+		///	<see cref="GetWebThread" />.
+		/// </remarks>
+		/// <param name="key">
+		///	Album key.
+		/// </param>
+		/// <param name="url">
+		///	The URL from which to get the cover.
+		/// </param>
+		/// <param name="done_func">
+		///	A <see cref="GotCoverDelegate" />.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> of the temporary cover.
+		/// </returns>
 		public Pixbuf GetWeb (string key, string url, GotCoverDelegate done_func)
 		{
 			db.RemoveCover (key);
@@ -294,12 +209,47 @@ namespace Muine
 
 
 		// Methods :: Public :: GetAmazon
+		/// <summary>
+		///	Search for the album cover on Amazon.
+		/// </summary>
+		/// <remarks>
+		///	Immediately returns a temporary cover.
+		///	The actual downloading occurs in
+		///	<see cref="GetAmazonThread" />.
+		/// </remarks>
+		/// <param name="album">
+		///	An <see cref="Album" />.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> of the temporary cover.
+		/// </returns>
 		public Pixbuf GetAmazon (Album album)
 		{
 			return GetAmazon (album, true);
 		}
 		
 		
+		/// <summary>
+		///	Search for the album cover on Amazon,
+		///	optionally marking the covering as being checked.
+		/// </summary>
+		/// <remarks>
+		///	Immediately returns a temporary cover.
+		///	The actual downloading occurs in
+		///	<see cref="GetAmazonThread" />.
+		/// </remarks>
+		/// <param name="album">
+		///	An <see cref="Album" />.
+		/// </param>
+		/// <param name="mark">
+		///	Whether or not to mark the cover in the database as
+		///	currently being downloaded. Normally, this should be
+		///	'true' but if you want to avoid modifying the database,
+		///	it should be 'false'.
+		/// </parm>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> of the temporary cover.
+		/// </returns>
 		public Pixbuf GetAmazon (Album album, bool mark)
 		{
 			if (mark)
@@ -311,6 +261,20 @@ namespace Muine
 		}
 
 		// Methods :: Public :: DownloadFromAmazon
+		//	TODO: Refactor this
+		/// <summary>
+		///	Search for the album cover on Amazon.
+		/// </summary>
+		/// <remarks>
+		///	This should only be called from <see cref="GetAmazonThread" />.
+		///	Normally, <see cref="GetAmazon" /> should be used instead.
+		/// </remarks>
+		/// <param name="album">
+		///	An <see cref="Album" />.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> if a cover is found, null otherwise.
+		/// </returns>
 		public Pixbuf DownloadFromAmazon (Album album)
 		{
 			Amazon.AmazonSearchService search_service = new Amazon.AmazonSearchService ();
@@ -318,7 +282,9 @@ namespace Muine
 			string sane_album_title = SanitizeString (album.Name);
 
 			// remove "disc 1" and family
-			sane_album_title =  Regex.Replace (sane_album_title, @"[,:]?\s*(cd|dis[ck])\s*(\d+|one|two|three|four|five|six|seven|eight|nine|ten)\s*$", "");
+			//	TODO: Make the regex translatable? (e.g. "uno|dos|tres...")
+			sane_album_title =  Regex.Replace (sane_album_title, 
+				@"[,:]?\s*(cd|dis[ck])\s*(\d+|one|two|three|four|five|six|seven|eight|nine|ten)\s*$", "");
 
 			string [] album_title_array = sane_album_title.Split (' ');
 			Array.Sort (album_title_array);
@@ -366,7 +332,7 @@ namespace Muine
 			Pixbuf best_match = null;
 
 			while (current_page <= total_pages && current_page <= max_pages) {
-				asearch.page = Convert.ToString (current_page);
+			       asearch.page = Convert.ToString (current_page);
 
 				Amazon.ProductInfo pi;
 				
@@ -424,6 +390,7 @@ namespace Muine
 
 						backward_match_count++;
 					}
+
 					backward_match_percent = backward_match_count / (double) product_name_array.Length;
 
 					double total_match_percent = match_percent + backward_match_percent;
@@ -464,6 +431,26 @@ namespace Muine
 		}
 
 		// Methods :: Public :: Download
+		/// <summary>
+		///	Get the cover from a URL.
+		/// </summary>
+		/// <remarks>
+		///	This should only be called from <see cref="GetWebThread" />
+		/// 	and <see cref="DownloadFromAmazon" />. Normally, 
+		///	<see cref="GetWeb" /> should be used instead.
+		/// </remarks>
+		/// <param name="url">
+		///	The URL from which to get the cover.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> if a cover is found, null otherwise.
+		/// </returns>
+		/// <exception cref="WebException">
+		///	Thrown if an error occurred while downloading.
+		/// </exception>
+		/// <exception cref="GLib.GException">
+		///	Thrown if loading file fails.
+		/// </exception>
 		public Pixbuf Download (string url)
 		{
 			Pixbuf cover;
@@ -477,8 +464,7 @@ namespace Muine
 				req.Proxy = proxy.Proxy;
 				
 			WebResponse resp = null;
-		
-			// May throw an exception, but we catch it in the calling function
+
 			resp = req.GetResponse ();
 
 			Stream s = resp.GetResponseStream ();
@@ -494,6 +480,16 @@ namespace Muine
 			return cover;
 		}
 
+		// Methods :: Public :: AddBorder
+		/// <summary>
+		///	Scale the image and add a black 1 pixel border to it.
+		/// </summary>
+		/// <param name="cover">
+		///	The <see cref="Gdk.Pixbuf" /> to modify.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> of the modified cover.
+		/// </returns>
 		public Pixbuf AddBorder (Pixbuf cover)
 		{
 			Pixbuf border;
@@ -519,7 +515,7 @@ namespace Muine
 			// create the background + black border pixbuf
 			border = new Pixbuf (Colorspace.Rgb, true, 8, cover.Width + 2, cover.Height + 2);
 			border.Fill (0x000000ff);
-				
+
 			// put the cover image on the border area
 			cover.CopyArea (0, 0, cover.Width, cover.Height, border, 1, 1);
 
@@ -529,6 +525,22 @@ namespace Muine
 
 		// Methods :: Private
 		// Methods :: Private :: SanitizeString
+		//	TODO: We should probably also trim the string at the 
+		//	end.
+		/// <summary>
+		///	Sanitize the string for use in <see cref="DownloadFromAmazon" />.
+		/// </summary>
+		/// <remarks>
+		///	The string is made lower-case, all text within 
+		///	parentheses or square brackets are discarded. Dashes,
+		///	underscores and plus signs are removed.
+		/// </remarks>
+		/// <param name="s">
+		///	The <see cref="String" /> which to modify.
+		/// </param>
+		/// <returns>
+		///	A <see cref="Gdk.Pixbuf" /> of the modified cover.
+		/// </returns>
 		private string SanitizeString (string s)
 		{
 			s = s.ToLower ();
@@ -543,9 +555,285 @@ namespace Muine
 
 		// Handlers
 		// Handlers :: OnAmazonLocaleChanged
+		/// <summary>
+		///	Handler called when the Amazon locale is changed in GConf.
+		/// </summary>
+		/// <param name="o">
+		///	The calling object.
+		/// </param>
+		/// <param name="args">
+		///	The <see cref="GConf.NotifyEventArgs" />.
+		/// </param>
 		private void OnAmazonLocaleChanged (object o, GConf.NotifyEventArgs args)
 		{
 			amazon_locale = (string) args.Value;
 		}
+
+		// Internal Classes
+		// Internal Classes :: GetWebThread
+		// 	FIXME: Split off? This is big.
+		private class GetWebThread
+		{
+			// Delegates
+			private GotCoverDelegate done_func;
+			
+			// Objects
+			private CoverGetter getter;
+			private Pixbuf pixbuf = null;
+
+			// Variables
+			private string key;
+			private string url;
+
+			// Constructor
+			/// <summary>
+			///	Create a new <see cref="GetWebThread" />.
+			/// </summary>
+			/// <param name="getter">
+			///	A <see cref="CoverGetter" />.
+			/// </param>
+			/// <param name="key">
+			/// 	The album key.
+			/// </param>
+			/// <param name="url">
+			///	The URL from which to download the cover.
+			/// </param>
+			/// <param name="done_func">
+			///	The delegate to call when the cover is done downloading.
+			/// </param>
+			/// <returns>
+			///	A <see cref="Gdk.Pixbuf" /> of the modified cover.
+			/// </returns>
+			public GetWebThread (CoverGetter getter, string key, string url,
+					     GotCoverDelegate done_func)
+			{
+				this.getter = getter;
+				this.key = key;
+				this.url = url;
+				this.done_func = done_func;
+
+				Thread thread = new Thread (new ThreadStart (ThreadFunc));
+				thread.IsBackground = true;
+				thread.Priority = ThreadPriority.BelowNormal;
+				thread.Start ();
+			}
+
+			// Methods
+			// Methods :: Private
+			// Methods :: Private :: SignalIdle
+			/// <summary>
+			///	Calls the <see cref="GotCoverDelegate" />.
+			/// </summary>
+			/// <remarks>
+			///	<para>
+			///	This is the method called when idling. It calls the 
+			///	<see cref="GotCoverDelegate" /> because if we're
+			///	idling, we must be done downloading.
+			///	</para>
+			///
+			///	<para>
+			/// 	This is not entirely safe, as the user can in theory
+			/// 	have changed the cover image between the thread and
+			/// 	this idle. However, chances are very, very slim and
+			/// 	things won't explode if it happens.
+			///	</para>
+			/// </remarks>
+			/// <returns>
+			///	False, we only want to do this once.
+			/// </returns>
+			private bool SignalIdle ()
+			{
+				done_func (pixbuf);
+
+				return false;
+			}			
+
+			// Delegate Functions
+			// Delegate Functions :: ThreadFunc
+			/// <summary>
+			///	Downloads the cover and sets it in the database.
+			/// </summary>
+			/// <remarks>
+			///	This is the main method of the thread.
+			/// </remarks>
+			private void ThreadFunc ()
+			{
+				try {
+					pixbuf = getter.Download (url);
+					pixbuf = getter.AddBorder (pixbuf);
+				} catch {
+				}
+
+				// Check if cover has been modified while we were downloading
+				if (Global.CoverDB.Covers [key] != null)
+					return;
+
+				if (pixbuf != null)
+					Global.CoverDB.SetCover (key, pixbuf);
+
+				// Also do this if it is null, as we need to remove the
+				// downloading image
+				GLib.IdleHandler idle = new GLib.IdleHandler (SignalIdle);
+				GLib.Idle.Add (idle);
+			}
+		}
+
+		// Internal Classes :: GetAmazonThread
+		// 	FIXME: Split off? This is big.
+		/// <summary>
+		///	This is used to download covers from Amazon in a threaded
+		///	fashion.
+		/// </summary>
+		/// <remarks>
+		///	To download a cover, simply <see cref="Queue.Enqueue" />
+		/// 	it on the <see cref="Queue" />.
+		/// </remarks>
+		private class GetAmazonThread
+		{
+			// Objects
+			private CoverGetter getter;
+			private Queue queue;
+
+			// Internal Classes
+			// Internal Classes :: IdleData
+			// 	FIXME: Internal classes inside internal classes? ick!
+			private class IdleData
+			{
+				// Objects
+				private Album album;
+				private Pixbuf pixbuf;
+
+				// Constructor
+				/// <summary>
+				///	Create a new <see cref="IdleData" /> object.
+				/// </summary>
+				public IdleData (Album album, Pixbuf pixbuf)
+				{
+					this.album = album;
+					this.pixbuf = pixbuf;
+
+					GLib.IdleHandler idle = new GLib.IdleHandler (IdleFunc);
+					GLib.Idle.Add (idle);
+				}
+
+				// Delegate Functions
+				// Delegate Functions :: IdleFunc
+				/// <summary>
+				///	Sets the cover to the currently held
+				///	<see cref="Gdk.Pixbuf" />.
+				/// </summary>
+				/// <remarks>
+				///	<para>
+				///	This is the method called when idling. It is used
+				///	to flush the covers in the queue to the database.
+				///	</para>
+				///
+				///	<para>
+				/// 	This is not entirely safe, as the user can in theory
+				/// 	have changed the cover image between the thread and
+				/// 	this idle. However, chances are very, very slim and
+				/// 	things won't explode if it happens.
+				///	</para>
+				/// </remarks>
+				/// <returns>
+				///	False, we only want to do this once.
+				/// </returns>
+				private bool IdleFunc ()
+				{
+					album.CoverImage = pixbuf;
+		
+					return false;
+				}
+			}
+
+			// Constructor
+			/// <summary>
+			///	Create a new <see cref="GetAmazonThread" />.
+			/// </summary>			
+			public GetAmazonThread (CoverGetter getter)
+			{
+				this.getter = getter;
+
+				queue = Queue.Synchronized (new Queue ());
+
+				Thread thread = new Thread (new ThreadStart (ThreadFunc));
+				thread.IsBackground = true;
+				thread.Priority = ThreadPriority.BelowNormal;
+				thread.Start ();
+			}
+
+			// Properties
+			// Properties :: Queue (get;)
+			/// <summary>
+			///	The <see cref="Queue" />, which holds albums to
+			///	download.
+			/// </summary>
+			/// <returns>
+			/// 	A <see cref="Queue" />.
+			/// </returns>
+			public Queue Queue {
+				get { return queue; }
+			}
+
+			// Delegate Functions
+			// Delegate Functions :: ThreadFunc
+			//	TODO: Refactor
+			/// <summary>
+			///	Downloads the cover and sets it in the database.
+			/// </summary>
+			/// <remarks>
+			///	<para>
+			///	This is the main method of the thread.
+			///	</para>
+			///
+			///	<para>
+			///	If the cover has trouble downloading, this thread
+			///	sleeps for a minute. In order to not overload the
+			///	server, this thread sleeps for a second between
+			///	downloads. The Amazon API requires that we do this.
+			///	</para>
+			/// </remarks>
+			private void ThreadFunc ()
+			{
+				while (true) {
+					while (queue.Count > 0) {
+						Album album = (Album) queue.Dequeue ();
+						Pixbuf pixbuf = null;
+
+						// Download Cover
+						try {
+							pixbuf = getter.DownloadFromAmazon (album);
+
+						} catch (WebException e) {
+							// Temporary web problem (Timeout etc.) - re-queue
+							Thread.Sleep (60000); // wait for a minute first
+							queue.Enqueue (album);
+							continue;
+
+						} catch (Exception e) {
+						}
+
+						string key = album.Key;
+						
+						// Check if cover has been modified while we were downloading
+						if (Global.CoverDB.Covers [key] != null)
+							continue;
+
+						// Add border and set cover						
+						if (pixbuf == null) {
+							Global.CoverDB.UnmarkAsBeingChecked (key);
+
+						} else {
+							pixbuf = getter.AddBorder (pixbuf);
+							Global.CoverDB.SetCover (key, pixbuf);
+						}
+
+						new IdleData (album, pixbuf);
+					}
+
+					Thread.Sleep (1000);
+				}
+			}						
+		}
 	}
 }
