@@ -33,22 +33,44 @@ namespace Muine
 		[Glade.Widget] private Label  label;
 
 		// Constructor
-		public ErrorDialog (PlaylistWindow playlist,
-				    string primary_text,
-				    string secondary_text)
-		: this (primary_text, secondary_text)
+		//	TODO: Shouldn't the ErrorDialog be able to be
+		//	associated with *any* window, not just the
+		//	PlaylistWindow?
+		/// <summary>
+		///	Create a new <see cref="ErrorDialog" />.
+		/// </summary>
+		/// <param name="playlist">
+		///	The <see cref="PlaylistWindow" /> which should be the
+		/// 	parent of this window.
+		/// </param>
+		/// <param name="s1">
+		///	The <see cref="String" /> to be used as the primary text.
+		/// </param>
+		/// <param name="s2">
+		///	The <see cref="String" /> to be used as the secondary text.
+		/// </param>
+		public ErrorDialog (PlaylistWindow playlist, string s1, string s2) : this (s1, s2)
 		{
 			window.TransientFor = playlist;
 		}
 
-		public ErrorDialog (string primary_text, string secondary_text)
+		/// <summary>
+		///	Create a new <see cref="ErrorDialog" />.
+		/// </summary>
+		/// <param name="s1">
+		///	The <see cref="String" /> to be used as the primary text.
+		/// </param>
+		/// <param name="s2">
+		///	The <see cref="String" /> to be used as the secondary text.
+		/// </param>
+		public ErrorDialog (string s1, string s2)
 		{
 			Glade.XML gxml = new Glade.XML (null, "ErrorDialog.glade", "window", null);
 			gxml.Autoconnect (this);
 
 			string full_text = String.Format ("<span size=\"large\" weight=\"bold\">{0}</span>\n\n{1}",
-			                                  StringUtils.EscapeForPango (primary_text),
-							  StringUtils.EscapeForPango (secondary_text));
+				StringUtils.EscapeForPango (s1),
+				StringUtils.EscapeForPango (s2));
 
 			label.Markup = full_text;
 
