@@ -31,6 +31,7 @@ namespace Muine
 		// Strings
 		private static readonly string string_title =
 			Catalog.GetString ("Importing \"{0}\"");
+
 		private static readonly string string_loading =
 			Catalog.GetString ("Loading:");
 
@@ -45,6 +46,7 @@ namespace Muine
 
 		// Variables
 		private bool canceled = false;
+		private Gdk.Geometry geo_no_resize_height;
 
 		// Constructor
 		public ProgressWindow (Window parent)
@@ -62,9 +64,12 @@ namespace Muine
 			file_label_container.Add (file_label);
 
 			loading_label.Markup = String.Format ("<b>{0}</b>",
-							      StringUtils.EscapeForPango (string_loading));
+				StringUtils.EscapeForPango (string_loading));
 
 			title_format = string_title;
+
+			geo_no_resize_height = new Gdk.Geometry ();
+			geo_no_resize_height.MaxWidth = Int32.MaxValue;
 		}
 
 		// Methods
@@ -106,5 +111,15 @@ namespace Muine
 			args.RetVal = true;
 			canceled = true;
 		}
+
+		// Handlers :: OnWindowSizeRequested
+		private void OnWindowSizeRequested (object o, SizeRequestedArgs args)
+		{
+			if (geo_no_resize_height.MaxHeight == args.Requisition.Height)
+				return;
+
+			geo_no_resize_height.MaxHeight = args.Requisition.Height;
+			window.SetGeometryHints (window, geo_no_resize_height, Gdk.WindowHints.MaxSize);
+		}
 	}
 }
