@@ -34,10 +34,13 @@ namespace Muine
 		[Glade.Widget] private Label  song_position;
 
 		// Objects
-		IPlayer player;
+		private IPlayer player;
 
 		// Variables
-		bool from_tick;
+		private bool from_tick;
+		private const uint set_position_timeout = 100;
+		private uint set_position_timeout_id;
+		private Gdk.Geometry geo_no_resize_height;
 		
 		// Constructor
 		public SkipToWindow (IPlayer p)
@@ -47,6 +50,9 @@ namespace Muine
 
 			window.TransientFor = p.Window;
 
+			geo_no_resize_height = new Gdk.Geometry ();
+			geo_no_resize_height.MaxWidth = Int32.MaxValue;
+
 			player = p;
 			player.TickEvent += new TickEventHandler (OnTickEvent);
 
@@ -68,39 +74,78 @@ namespace Muine
 			window.Visible = false;
 		}
 
+		// Methods :: Private
+		// Methods :: Private :: SetPositionTimeoutFunc
+		private bool SetPositionTimeoutFunc ()
+		{
+			set_position_timeout_id = 0;
+			player.Position = (int) song_slider.Value;
+			
+			return false;
+		}
+
+		// Methods :: Private :: UpdateLabel
+		private void UpdateLabel (int pos)
+		{
+			String position   = StringUtils.SecondsToString (pos);
+			String total_time = StringUtils.SecondsToString (player.PlayingSong.Duration);
+			song_position.Text = String.Format ("{0} / {1}", position, total_time);
+		}
+
 		// Handlers
 		// Handlers :: OnTickEvent
 		private void OnTickEvent (int pos) 
 		{
-			// Update label
-			String position   = StringUtils.SecondsToString (pos);
-			String total_time = StringUtils.SecondsToString (player.PlayingSong.Duration);
-			song_position.Text = String.Format ("{0} / {1}", position, total_time);
+			if (set_position_timeout_id > 0)
+				return;
+
+			UpdateLabel (pos);
 
 			// Update slider
 			from_tick = true;
 			song_slider.SetRange (0, player.PlayingSong.Duration);
+
+			if (pos > player.PlayingSong.Duration)
+				return;
+
 			song_slider.Value = pos; 
 		}
 
-		// Handlers :: OnSongSliderValueChanged
+		// Handlers :: OnSongSliderValueChanged (Glade)
 		private void OnSongSliderValueChanged (object o, EventArgs a) 
 		{
-			if (!from_tick)
-				player.Position = (int) song_slider.Value;
-			else
+			if (from_tick) {
 				from_tick = false;
+				return;
+			}
+
+			if (set_position_timeout_id > 0)
+				GLib.Source.Remove (set_position_timeout_id);
+
+			set_position_timeout_id = GLib.Timeout.Add (set_position_timeout,
+				new GLib.TimeoutHandler (SetPositionTimeoutFunc));
+
+			UpdateLabel ((int) song_slider.Value);
 		}
 
 		// Handlers :: OnWindowDeleteEvent
 		private void OnWindowDeleteEvent (object o, EventArgs a)
 		{
-			window.Visible = false;
-			
+			window.Visible = false;			
 			DeleteEventArgs args = (DeleteEventArgs) a;
 			args.RetVal = true;
 		}
 
+		// Handlers :: OnWindowSizeRequested
+		private void OnWindowSizeRequested (object o, SizeRequestedArgs args)
+		{
+			if (geo_no_resize_height.MaxHeight == args.Requisition.Height)
+				return;
+
+			geo_no_resize_height.MaxHeight = args.Requisition.Height;
+			window.SetGeometryHints (window, geo_no_resize_height, Gdk.WindowHints.MaxSize);
+		}
+
 		// Handlers :: OnCloseButtonClicked
 		private void OnCloseButtonClicked (object o, EventArgs a)
 		{
