@@ -26,7 +26,7 @@ namespace Muine
 	public class VolumeButton : ToggleButton
 	{
 		// Constants
-		// 	GDK_CURRENT_TIME doesn't seem to have an equiv in gtk# yet.
+		// 	TODO: GDK_CURRENT_TIME doesn't seem to have an equiv in gtk# yet.
 		private const uint CURRENT_TIME = 0;
 
 		// Events
@@ -56,7 +56,7 @@ namespace Muine
 			base.ScrollEvent += new ScrollEventHandler (OnScrollEvent);
 			base.Toggled     += new EventHandler       (OnToggled    );
 			
-			base.Flags |= (int) WidgetFlags.NoWindow;
+			base.WidgetFlags |= WidgetFlags.NoWindow;
 		}
 
 		// Destructor
@@ -72,16 +72,11 @@ namespace Muine
 				volume = value;
 
 				string id = "muine-volume-";
-						
-				id += (volume <= 0)
-				      ? "zero"
-				      :
-				      (volume <= 100 / 3)
-				      ? "min"
-				      :
-				      (volume <= 200 / 3)
-				      ? "medium"
-				      : "max";
+				
+				if      (volume <= 000/3) id += "zero"  ;
+				else if (volume <= 100/3) id += "min"   ;
+				else if (volume <= 200/3) id += "medium";
+				else                      id += "max"   ;
 
 				icon.SetFromStock (id, IconSize.LargeToolbar);
 
@@ -98,45 +93,53 @@ namespace Muine
 		{
 			revert_volume = this.Volume;
 
+			// Popup
 			popup = new Window (WindowType.Popup);
 			popup.Screen = base.Screen;
+			popup.ButtonPressEvent += new ButtonPressEventHandler (OnPopupButtonPressEvent);
 
+			// Frame
 			Frame frame = new Frame ();
 			frame.Shadow = ShadowType.Out;
 			frame.Show ();
 
 			popup.Add (frame);
 
+			// Box
 			VBox box = new VBox (false, 0);
 			box.Show();
-
 			frame.Add (box);
 
+			// +
+			Gtk.Label plus_label = new Gtk.Label ("+");
+			plus_label.Show ();
+			box.PackStart (plus_label, false, true, 0);
+
+			// Adjustment
 			Adjustment adj = new Adjustment (volume, 0, 100, 5, 10, 0);		
 
+			// Scale
 			VScale scale = new VScale (adj);
-			scale.ValueChanged += new EventHandler (OnScaleValueChanged);
+
+			scale.ValueChanged  += new EventHandler         (OnScaleValueChanged );
 			scale.KeyPressEvent += new KeyPressEventHandler (OnScaleKeyPressEvent);
-			popup.ButtonPressEvent += new ButtonPressEventHandler (OnPopupButtonPressEvent);
 
 			scale.Adjustment.Upper = 100.0;
-			scale.Adjustment.Lower = 0.0;
+			scale.Adjustment.Lower =   0.0;
 			scale.DrawValue = false;
+			scale.Inverted  = true ;
 			scale.UpdatePolicy = UpdateType.Continuous;
-			scale.Inverted = true;
 
 			scale.Show ();
 
-			Label label = new Label ("+");
-			label.Show ();
-			box.PackStart (label, false, true, 0);
-
-			label = new Label ("-");
-			label.Show ();
-			box.PackEnd (label, false, true, 0);
-
 			box.PackStart (scale, true, true, 0);
 
+			// -
+			Gtk.Label minus_label = new Gtk.Label ("-");
+			minus_label.Show ();
+			box.PackEnd (minus_label, false, true, 0);
+
+
 			Requisition req = SizeRequest ();
 
 			int x, y;
@@ -153,79 +156,83 @@ namespace Muine
 			Grab.Add (popup);
 
 			Gdk.GrabStatus grabbed = Gdk.Pointer.Grab (popup.GdkWindow, true, 
-								   Gdk.EventMask.ButtonPressMask | Gdk.EventMask.ButtonReleaseMask | Gdk.EventMask.PointerMotionMask, 
-								   null, null, 
-								   CURRENT_TIME);
+				Gdk.EventMask.ButtonPressMask   | 
+				Gdk.EventMask.ButtonReleaseMask | 
+				Gdk.EventMask.PointerMotionMask, 
+				null, null, CURRENT_TIME);
 
 			if (grabbed == Gdk.GrabStatus.Success) {
 				grabbed = Gdk.Keyboard.Grab (popup.GdkWindow, true, CURRENT_TIME);
 
-				if (grabbed != Gdk.GrabStatus.Success) {
-					Grab.Remove (popup);
-					popup.Destroy ();
-					popup = null;
-				}
-			} else {
+				if (grabbed == Gdk.GrabStatus.Success)
+					return;
+					
 				Grab.Remove (popup);
 				popup.Destroy ();
 				popup = null;
+
+				return;
 			}
+
+			Grab.Remove (popup);
+			popup.Destroy ();
+			popup = null;
 		}
 
 		// Methods :: Private :: HideScale
 		private void HideScale ()
 		{
-			if (popup != null) {
-				Grab.Remove (popup);
-				Gdk.Pointer.Ungrab (CURRENT_TIME);
-				Gdk.Keyboard.Ungrab (CURRENT_TIME);
-
-				popup.Destroy ();
-				popup = null;
-			}
-
 			Active = false;
+
+			if (popup == null)
+				return;
+
+			Grab.Remove (popup);
+			Gdk.Pointer .Ungrab (CURRENT_TIME);
+			Gdk.Keyboard.Ungrab (CURRENT_TIME);
+
+			popup.Destroy ();
+			popup = null;
 		}
 
 		// Handlers
 		// Handlers :: OnToggled
 		private void OnToggled (object obj, EventArgs args)
 		{
-			if (Active)
-				ShowScale ();
-			else
-				HideScale ();
+			if (Active) ShowScale ();
+			else        HideScale ();
 		}
 
 		// Handlers :: OnScrollEvent
 		private void OnScrollEvent (object obj, ScrollEventArgs args)
 		{
-			int tmp_vol = Volume;
+			int v = this.Volume;
 			
 			switch (args.Event.Direction) {
 			case Gdk.ScrollDirection.Up:
-				tmp_vol += 10;
+				v += 10;
 				break;
 
 			case Gdk.ScrollDirection.Down:
-				tmp_vol -= 10;
+				v -= 10;
 				break;
 
 			default:
 				break;
 			}
 
-			// Assure volume is between 0 and 100
-			tmp_vol = Math.Min (100, tmp_vol);
-			tmp_vol = Math.Max (  0, tmp_vol);
+			// Ensure volume is between 0 and 100
+			v = Math.Min (100, v);
+			v = Math.Max (  0, v);
 
-			this.Volume = tmp_vol;
+			this.Volume = v;
 		}
 
 		// Handlers :: OnScaleValueChanged
 		private void OnScaleValueChanged (object obj, EventArgs args)
 		{
 			VScale scale = (VScale) obj;
+
 			this.Volume = (int) scale.Value;
 		}
 
@@ -236,6 +243,7 @@ namespace Muine
 			case Gdk.Key.Escape:
 				HideScale ();
 				this.Volume = revert_volume;
+
 				break;
 
 			case Gdk.Key.KP_Enter:
@@ -245,6 +253,7 @@ namespace Muine
 			case Gdk.Key.space:
 			case Gdk.Key.KP_Space:
 				HideScale ();
+
 				break;
 
 			default:
@@ -255,8 +264,10 @@ namespace Muine
 		// Handlers :: OnPopupButtonPressEvent
 		private void OnPopupButtonPressEvent (object obj, ButtonPressEventArgs args)
 		{
-			if (popup != null)
-				HideScale ();
+			if (popup == null)
+				return;
+
+			HideScale ();
 		}
 	}
 }
